/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */



#include "Dialogs/CustomNoteSelectDialog.h"
#include "GUI/GraphicalTrack.h"
#include "Midi/Sequence.h"
#include "Midi/Track.h"
#include "GUI/GLPane.h"

namespace AriaMaestosa {
	
enum {
 id_ok,
 id_cancel    
};

BEGIN_EVENT_TABLE(CustomNoteSelectDialog, wxDialog)

EVT_BUTTON(id_ok, CustomNoteSelectDialog::okClicked)
EVT_BUTTON(id_cancel, CustomNoteSelectDialog::cancelClicked)
EVT_KEY_DOWN(CustomNoteSelectDialog::keyPress)

END_EVENT_TABLE()


CustomNoteSelectDialog::CustomNoteSelectDialog() : wxDialog(NULL, wxID_ANY, wxGetTranslation("Select notes..."), wxPoint(300,100), wxSize(200,400))
{
	INIT_LEAK_CHECK();
    
    wxSize smallTextCtrlSize(wxDefaultSize);
    smallTextCtrlSize.SetWidth(35);
    
    boxSizer=new wxBoxSizer(wxVERTICAL);
    
    wxStaticText* txt=new wxStaticText(this, wxID_ANY, wxGetTranslation("Select notes that..."));
    boxSizer->Add(txt, 0, wxALL, 15);
    
    cb_pitch=new wxCheckBox(this, wxID_ANY, wxGetTranslation("have the same pitch"));
    boxSizer->Add(cb_pitch, 0, wxALL, 5);
    

    // --------------------------------------
    {
    wxPanel* panel_volume=new wxPanel(this);
    boxSizer->Add(panel_volume, 0, wxALL, 0);
    wxBoxSizer* subBoxSizer=new wxBoxSizer(wxHORIZONTAL);
    
    cb_volume=new wxCheckBox(panel_volume, wxID_ANY, wxGetTranslation("have a similar volume"));
    subBoxSizer->Add(cb_volume, 0, wxALL, 5);
    
    wxStaticText* lbl_more_or_less = new wxStaticText(panel_volume, wxID_ANY, wxT("+/-"));
    subBoxSizer->Add(lbl_more_or_less, 00, wxALL, 5);
    
    volume_tolerance = new wxTextCtrl(panel_volume, wxID_ANY, wxT("5"), wxDefaultPosition, smallTextCtrlSize);
    subBoxSizer->Add(volume_tolerance, 0, wxALL, 5);
    
    panel_volume->SetAutoLayout(TRUE);
    panel_volume->SetSizer(subBoxSizer);
    subBoxSizer->Layout();
    }
    // --------------------------------------
    
    cb_string=new wxCheckBox(this, wxID_ANY, wxGetTranslation("are on the same string"));
    boxSizer->Add(cb_string, 0, wxALL, 5);
    
    cb_fret=new wxCheckBox(this, wxID_ANY, wxGetTranslation("have the same fret"));
    boxSizer->Add(cb_fret, 0, wxALL, 5);
    
    // --------------------------------------
    {
        wxPanel* panel_duration=new wxPanel(this);
        boxSizer->Add(panel_duration, 0, wxALL, 0);
        wxBoxSizer* subBoxSizer=new wxBoxSizer(wxHORIZONTAL);
        
        cb_duration=new wxCheckBox(panel_duration, wxID_ANY, wxGetTranslation("have a similar duration"));
        subBoxSizer->Add(cb_duration, 0, wxALL, 5);
		
        panel_duration->SetAutoLayout(TRUE);
        panel_duration->SetSizer(subBoxSizer);
        subBoxSizer->Layout();
    }
    // --------------------------------------
    
    // --------------------------------------
    {
    wxPanel* panel_measures=new wxPanel(this);
    boxSizer->Add(panel_measures, 0, wxALL, 0);
    wxBoxSizer* subBoxSizer=new wxBoxSizer(wxHORIZONTAL);
    
    cb_measure=new wxCheckBox(panel_measures, wxID_ANY, wxGetTranslation("are located in measures "));
    subBoxSizer->Add(cb_measure, 0, wxALL, 5);
    
    from_measure = new wxTextCtrl(panel_measures, wxID_ANY, wxT("1"), wxDefaultPosition, smallTextCtrlSize);
    subBoxSizer->Add(from_measure, 0, wxALL, 5);

    wxStaticText* lbl_to = new wxStaticText(panel_measures, wxID_ANY, wxGetTranslation(" to "));
    subBoxSizer->Add(lbl_to, 0, wxALL, 5);
    
    to_measure = new wxTextCtrl(panel_measures, wxID_ANY, wxT("2"), wxDefaultPosition, smallTextCtrlSize);
    subBoxSizer->Add(to_measure, 0, wxALL, 5);
    
    panel_measures->SetAutoLayout(TRUE);
    panel_measures->SetSizer(subBoxSizer);
    subBoxSizer->Layout();
    }
    // --------------------------------------
    
    // --------------------------------------
    {
    wxPanel* panel_ok_cancel=new wxPanel(this);
    boxSizer->Add(panel_ok_cancel, 0, wxALL, 0);
    wxBoxSizer* subBoxSizer=new wxBoxSizer(wxHORIZONTAL);
    
    wxButton* okbtn=new wxButton(panel_ok_cancel, id_ok, wxT("OK"));
    subBoxSizer->Add(okbtn, 0, wxALL, 15);
	
    okbtn->SetDefault();
	
    wxButton* cancelbtn=new wxButton(panel_ok_cancel, id_cancel, wxGetTranslation("Cancel"));
    subBoxSizer->Add(cancelbtn, 0, wxALL, 15);
        
    panel_ok_cancel->SetAutoLayout(TRUE);
    panel_ok_cancel->SetSizer(subBoxSizer);
    subBoxSizer->Layout();
    }
    // --------------------------------------
    
    SetAutoLayout(TRUE);
    SetSizer(boxSizer);
    boxSizer->Layout();
    boxSizer->SetSizeHints(this);
}

void CustomNoteSelectDialog::show(Track* currentTrack)
{
    CustomNoteSelectDialog::currentTrack = currentTrack;
    
    if(currentTrack->graphics->editorMode == GUITAR)
	{
        cb_string->Enable(true);
        cb_fret->Enable(true);
    }
	else
	{
        cb_string->Enable(false);
        cb_fret->Enable(false);
    }
    returnCode = ShowModal();   
}

void CustomNoteSelectDialog::keyPress(wxKeyEvent& evt)
{
    
    if(evt.GetKeyCode() == WXK_RETURN)
	{
        wxCommandEvent unused;
        okClicked(unused);
    }
}

void CustomNoteSelectDialog::okClicked(wxCommandEvent& evt)
{
    EndModal(returnCode);
    
    const bool pitch = cb_pitch->IsChecked();
    const bool volume = cb_volume->IsChecked();
    const bool string = cb_string->IsChecked() and currentTrack->graphics->editorMode == GUITAR;
    const bool fret = cb_fret->IsChecked() and currentTrack->graphics->editorMode == GUITAR;
    const bool duration = cb_duration->IsChecked();
    const bool measure = cb_measure->IsChecked();
    
    const int from_measure_value = atoi( (char*)(from_measure->GetValue().c_str()) );
    const int to_measure_value = atoi( (char*)(to_measure->GetValue().c_str()) );
    const int volume_tolerance_value = atoi( (char*)volume_tolerance->GetValue().c_str() );
    const int duration_tolerance_value = 0;
    
    // collect information about currently selected notes, to be then used for finding similar notes
    int referenceNoteAmount=0;
    const int noteAmount = currentTrack->getNoteAmount();
    for(int n=0; n<noteAmount; n++)
	{
        if(currentTrack->isNoteSelected(n)) referenceNoteAmount++;
    }
	
    int referencePitches[referenceNoteAmount];
    int referenceVolumes[referenceNoteAmount];
    int referenceStrings[referenceNoteAmount];
    int referenceFrets[referenceNoteAmount];
    int referenceDurations[referenceNoteAmount];
    
    int currentID=0;
    for(int n=0; n<noteAmount; n++)
	{
        if(currentTrack->isNoteSelected(n))
		{
            referencePitches[currentID] = currentTrack->getNotePitchID(n);
            referenceVolumes[currentID] = currentTrack->getNoteVolume(n);
            referenceStrings[currentID] = currentTrack->getNoteString(n);
            referenceFrets[currentID] = currentTrack->getNoteFret(n);
            referenceDurations[currentID] = currentTrack->getNoteEndInMidiTicks(n) - currentTrack->getNoteStartInMidiTicks(n);
            currentID++;
        }
    }
    
    currentTrack->selectNote(ALL_NOTES, false, true); // deselect all currently selected notes

    // test all notes one by one
    for(int n=0; n<noteAmount; n++)
	{
        
        // test for pitch, if pitch checkbox was checked
        if(pitch)
		{
            bool passTest = false;
            
            const int test_value = currentTrack->getNotePitchID(n);
            for(int i=0; i<referenceNoteAmount; i++)
			{
                if(test_value == referencePitches[i]) passTest=true;
            }
            
            if(!passTest) continue;
        }
        
        // test for string, if string checkbox was checked
        if(string)
		{
            bool passTest = false;
            
            const int test_value = currentTrack->getNoteString(n);
            for(int i=0; i<referenceNoteAmount; i++)
			{
                if(test_value == referenceStrings[i]) passTest=true;
            }
            
            if(!passTest) continue;
        }
        
        // test for fret, if fret checkbox was checked
        if(fret)
		{
            bool passTest = false;
            
            const int test_value = currentTrack->getNoteFret(n);
            for(int i=0; i<referenceNoteAmount; i++){
                if(test_value == referenceFrets[i]) passTest=true;
            }
            
            if(!passTest) continue;
        }

        // test for volume, if volume checkbox was checked
        if(volume)
		{
            bool passTest = false;
            
            const int test_value = currentTrack->getNoteVolume(n);
            for(int i=0; i<referenceNoteAmount; i++)
			{
                if( abs(test_value - referenceVolumes[i]) <= volume_tolerance_value ) passTest=true;
            }
            
            if(!passTest) continue;
        }
        
        // test for duration, if duration checkbox was checked
        if(duration)
		{
            bool passTest = false;
            
            const int test_value = currentTrack->getNoteEndInMidiTicks(n) - currentTrack->getNoteStartInMidiTicks(n);
            for(int i=0; i<referenceNoteAmount; i++)
			{
                if( abs(test_value - referenceDurations[i]) <= duration_tolerance_value ) passTest=true;
            }
            
            if(!passTest) continue;
        }
        
        // test for measure, if measure checkbox was checked
        if(measure)
		{
            bool passTest = false;
            
            // find in which measure the note is
            int test_value = currentTrack->getNoteStartInMidiTicks(n);
            test_value = (test_value - test_value % currentTrack->sequence->ticksPerMeasure()) / currentTrack->sequence->ticksPerMeasure();

            if(test_value >= from_measure_value-1 and test_value <= to_measure_value-1) passTest=true;

            if(!passTest) continue;
        }
        
        // if the flow reaches this part, it's because all checked tests succeeded. Select the note.
        currentTrack->selectNote(n, true, true);
        
    }//next
    
    currentTrack->sequence->glPane->render();
}

void CustomNoteSelectDialog::cancelClicked(wxCommandEvent& evt){
    EndModal(returnCode);
}

}
